# Extracting group information with one liners

This document lists a few one liners to extract useful group information from the JSON output of `collect_forks.py`.

* Find the number of groups
```sh
$ jq 'length' forks.json

20
```

* Get all groups with fewer than 3 students

```sh
$ jq '
  .[] \
  | {"group": .group, "size": (.members | length)} \
  | select(.size < 3)' forks.json

{
  "group": "group-w",
  "size": 1
}
```

* Find all students present in multiple groups

```sh
$ jq '.[].members[].name' forks.json \
  | sort \
  | uniq -dc

2 "Student McStudent"
```

* Find students without group (using an external list of names)

```sh
# 1. Get the list of students in groups, sort it
# 2. Sort the list of all enrolled students
# 3. Find names which are in enrolled students, but not grouped students
$ jq '.[].members[].name' forks.json \
  | sort \
  | comm -23 <( sort enrolled_students.txt) -

"John Smith"
```

* Combine forks from two different runs into a single json (for groups present in both files, the entry from run1.json is selected).

```sh
jq -s '.[0] + .[1] | unique_by(.group)' run1.json run2.json > both_runs.json
```

* Make a CSV table with one row per group from the flat JSON fields:

```sh
jq -r '.[] | [.id,.group,.url,.autostart_url] | @csv' forks.json
```

* Make a CSV table with one line per student:

```sh
jq -r 'map({user: .members[].name, group: .group}) | map([.user, .group])[] | @csv' forks.json
```
